use std::io::Read;
use std::process::Command;

use dbus::blocking::Connection;
use dbus::Message;

use crate::img_processors::process_image;
use crate::result::MBGResult;
use crate::TestingPic;

pub fn reset_background_handler(_: (), _: &Connection, message: &Message) -> bool {
    let member_name = message.read1::<String>();
    if let Ok(name) = member_name {
        if name.starts_with("org.mpris.MediaPlayer2.") {
            nitrogen_restore();
        }
    }
    true
}

pub fn nitrogen_restore() {
    info!("Reset BG");
    let out = Command::new("nitrogen").arg("--restore").output();
    if let Err(restore_error) = out {
        error!("Can't restore background: {}", restore_error.to_string());
    }
}

pub fn process_image_url(image_url: String) -> MBGResult<()> {
    let img_bytes = reqwest::blocking::get(image_url.as_str())?.bytes()?;
    let bytes_vec = img_bytes.into_iter().collect::<Vec<u8>>();
    let res = process_image(bytes_vec)?;
    let img_path = "/dev/shm/test.png";
    res.save(img_path)?;
    set_wallpaper(img_path);
    Ok(())
}

pub fn test_processors(test_data: TestingPic) -> MBGResult<()> {
    let mut img_file = std::fs::File::open(test_data.input.as_str())?;
    let mut image_bytes = Vec::new();
    img_file.read_to_end(&mut image_bytes)?;
    let res = process_image(image_bytes)?;
    res.save(test_data.output.as_str())?;
    if test_data.wallpaper {
        set_wallpaper(test_data.output.as_str());
    }
    Ok(())
}

pub fn set_wallpaper(path: &str) {
    debug!("Setting background");
    let out = Command::new("feh").arg("--bg-fill").arg(path).output();
    if let Err(set_error) = out {
        error!("Can't set background: {}", set_error.to_string());
    }
}
