#[macro_use]
extern crate failure;
#[macro_use]
extern crate failure_derive;
#[macro_use]
extern crate log;
#[macro_use]
extern crate serde_derive;

use structopt::StructOpt;

use crate::background::test_processors;
use crate::config::Config;
use crate::logging::setup_logger;
use crate::player_dbus::start_listen;
use crate::result::{MBGError, MBGResult};
use structopt::clap::Shell;

pub mod background;
pub mod config;
pub mod dbus_interface;
pub mod display;
pub mod img_processors;
pub mod logging;
pub mod player_dbus;
pub mod result;

include!("cli.rs");

fn main() -> MBGResult<()> {
    let opt: Opt = Opt::from_args();
    let logger = setup_logger();
    if let Err(error) = logger {
        error!("Failed to create pretty logger");
        error!("{}", error.to_string());
        error!("Using default logging");
    }
    let res = gtk::init();
    if res.is_err() {
        return Err(MBGError::X11Error(String::from("Could not connect to GTK")));
    }
    run(opt)
}
